static char help[] = "Loosing nullspaces in fieldsplit after zeroing rows.\n";

#include <petscsys.h>
#include <petscmat.h>
#include <petscksp.h>
#include <petscviewer.h>

int main(int argc, char **args)
{
  KSP         ksp;
  Vec         x;
  Vec         b;
  PetscViewer viewer;

  PetscInt rows;
  PetscInt cols;
  PetscInt size;
  
  PetscFunctionBeginUser;
  PetscCall(PetscInitialize(&argc, &args, (char *)0, help));

  PetscCall(PetscViewerBinaryOpen(PETSC_COMM_WORLD, "A.bin", FILE_MODE_READ, &viewer));
  Mat A;
  PetscCall(MatCreate(PETSC_COMM_WORLD, &A));
  PetscCall(MatLoad(A, viewer));
  PetscCall(PetscViewerDestroy(&viewer));

  MatNullSpace nullsp;
  PetscCall(MatGetNearNullSpace(A, &nullsp));
  PetscCall(MatGetSize(A, &rows, &cols));
  
  PetscInt bs;
  PetscCall(MatGetBlockSize(A, &bs));
  PetscCall(PetscPrintf(PETSC_COMM_WORLD, "A has rows = %d, cols = %d, bs = %d, nearnullsp = %p\n", rows, cols, bs, nullsp));

  PetscCall(PetscViewerBinaryOpen(PETSC_COMM_WORLD, "is.bin", FILE_MODE_READ, &viewer));

  IS is_thermal;
  PetscCall(ISCreate(PETSC_COMM_WORLD, &is_thermal));
  PetscCall(ISLoad(is_thermal, viewer));
  PetscCall(ISGetSize(is_thermal, &size));
  PetscCall(PetscPrintf(PETSC_COMM_WORLD, "thermal field size = %d\n", size));

  IS is_mech;
  PetscCall(ISCreate(PETSC_COMM_WORLD, &is_mech));
  PetscCall(ISLoad(is_mech, viewer));
  PetscCall(ISGetSize(is_mech, &size));
  PetscCall(PetscPrintf(PETSC_COMM_WORLD, "mechanical field size = %d\n", size));
  PetscCall(PetscViewerDestroy(&viewer));

  PetscCall(MatCreateVecs(A, &x, &b));
  PetscCall(VecZeroEntries(x));
  PetscCall(VecZeroEntries(b));
  
  PetscCall(KSPCreate(PETSC_COMM_WORLD, &ksp));
  PetscCall(KSPSetOperators(ksp, A, A));

  PetscCall(KSPSetType(ksp, KSPPREONLY));
  PetscCall(KSPSetTolerances(ksp, 1e-6, PETSC_DEFAULT, PETSC_DEFAULT, PETSC_DEFAULT));

  PC pc;
  PetscCall(KSPGetPC(ksp, &pc));
  PetscCall(PCSetType(pc, PCFIELDSPLIT));
  PetscCall(PCFieldSplitSetIS(pc, "thermal", is_thermal));
  PetscCall(PCFieldSplitSetIS(pc, "mechanical", is_mech));
  PetscCall(PCSetUp(pc));
  
  PetscInt n_splits = 0;
  KSP *sub_ksp;
  PetscCall(PCFieldSplitGetSubKSP(pc, &n_splits, &sub_ksp));
  PetscCall(PetscPrintf(PETSC_COMM_WORLD, "n_splits = %d\n", n_splits));
  
  PetscCall(KSPSetType(sub_ksp[0], KSPGMRES));
  PC pc_thermal;
  PetscCall(KSPGetPC(sub_ksp[0], &pc_thermal));
  PetscCall(PCSetType(pc_thermal, PCJACOBI));
  PetscCall(KSPSetFromOptions(sub_ksp[0]));

  PetscCall(KSPSetType(sub_ksp[1], KSPCG));
  PC pc_mech;
  PetscCall(KSPGetPC(sub_ksp[1], &pc_mech));
  PetscCall(PCSetType(pc_mech, PCGAMG));
  PetscCall(KSPSetFromOptions(sub_ksp[1]));
  
  Vec rigid_mode[6];
  PetscCall(PetscViewerBinaryOpen(PETSC_COMM_WORLD, "rigid-modes.bin", FILE_MODE_READ, &viewer));
  for (int i = 0; i < 6; i++) {
    PetscCall(VecCreate(PETSC_COMM_WORLD, &rigid_mode[i]));
    PetscCall(VecLoad(rigid_mode[i], viewer));
  }
  PetscCall(PetscViewerDestroy(&viewer));
  
  MatNullSpace near_null_space;
  PetscCall(MatNullSpaceCreate(PETSC_COMM_WORLD, PETSC_FALSE, 6, rigid_mode, &near_null_space));
  Mat K;
  PetscCall(KSPGetOperators(sub_ksp[1], &K, PETSC_NULLPTR));
  PetscCall(MatSetNearNullSpace(K, near_null_space));
  PetscCall(MatSetBlockSize(K, 3));

  PetscCall(MatGetSize(K, &rows, &cols));
  PetscCall(MatGetBlockSize(K, &bs));
  PetscCall(MatGetNearNullSpace(K, &nullsp));
  PetscCall(PetscPrintf(PETSC_COMM_WORLD, "K has rows = %d, cols = %d, bs = %d, nearnullsp = %p\n", rows, cols, bs, nullsp));
  
  const PetscInt *bc_thermal_indexes;
  PetscCall(ISGetIndices(is_thermal, &bc_thermal_indexes));

  const PetscInt *bc_mech_indexes;
  PetscCall(ISGetIndices(is_mech, &bc_mech_indexes));
  
// with columns it keeps the near nullspace
// without it it looses the near nullspace
  
  
  PetscBool has_columns = PETSC_FALSE;
  PetscCall(PetscOptionsHasName(PETSC_NULLPTR, PETSC_NULLPTR, "-columns", &has_columns));
  
  if (has_columns == PETSC_TRUE) {  
    PetscCall(MatZeroRowsColumns(A, 3, bc_mech_indexes, 1, PETSC_NULLPTR, PETSC_NULLPTR));
    PetscCall(MatZeroRowsColumns(A, 1, bc_thermal_indexes, 1, PETSC_NULLPTR, PETSC_NULLPTR));
  } else {
    PetscCall(MatZeroRows(A, 3, bc_mech_indexes, 1, PETSC_NULLPTR, PETSC_NULLPTR));
    PetscCall(MatZeroRows(A, 1, bc_thermal_indexes, 1, PETSC_NULLPTR, PETSC_NULLPTR));
  }
  
  PetscCall(ISRestoreIndices(is_mech, &bc_mech_indexes));
  PetscCall(ISRestoreIndices(is_thermal, &bc_thermal_indexes));

 
  
  PetscCall(KSPSetFromOptions(ksp));
  PetscCall(KSPSetUp(ksp));
  PetscCall(KSPSolve(ksp, b, x));

  // TODO: destroy
  PetscCall(PetscFinalize());
  return 0;
}
